﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace TaskGenNet_Sample_Code {

    public partial class frmMain : Form {

        #region member variables

        private TaskGenNet.Controller m_task = new TaskGenNet.Controller();
        private TaskGenNet.Controller.uDeviceInfo m_devInfo = new TaskGenNet.Controller.uDeviceInfo();
        private TaskGenNet.Controller.uBarcodeInfo BarcodeInfo = new TaskGenNet.Controller.uBarcodeInfo();

        private List<int> m_Record = new List<int>();
        private List<int> m_Field = new List<int>();
        private bool changeinfo = true;
        public string strTemplete = "";

        #endregion

        #region methods

        public frmMain()
        {
            InitializeComponent();
        }
                
        private void frmMain_Load(object sender, EventArgs e) {
            Initialize();
            tmrClock.Enabled = true;
            changeinfo = false;
        }

        private void chkLongPara_CheckedChanged(object sender, EventArgs e) {
            gbxComport.Width = (chkLongPara.Checked == true) ? 769 : 313;
        }

        private void ComportParaChanged_SelectedIndexChanged(object sender, EventArgs e) {
            if (changeinfo == true) return;

            int port = cbxComport.SelectedIndex;
            int baudrate = int.Parse(cbxBaudrate.Text);
            System.IO.Ports.Parity parity = (System.IO.Ports.Parity)cbxParity.SelectedIndex;
            int databit = int.Parse(cbxDataBits.Text);
            System.IO.Ports.StopBits stopbit = (System.IO.Ports.StopBits)cbxStopBits.SelectedIndex; 

            m_task.SerialPortSetting(port, baudrate, parity, databit, stopbit);
        }

        private void tmrClock_Tick(object sender, EventArgs e) {
            tssClock.Text = DateTime.Now.ToString("yyyy/MM/dd HH:mm:ss");
        }

        private void tbcMain_SelectedIndexChanged(object sender, EventArgs e) {
            switch (tbcMain.SelectedIndex) {
                case 0:
                    gbxGetDeviceSettings.Enabled = false;
                    break;
                case 1:
                    tbcBarcode.Enabled = false;
                    tbcBarcode.SelectedIndex = 0;
                    break;
                case 2:
                    gbxReceiveData.Enabled = false;
                    break;
            }
        }

        #endregion

        #region functions

        private void Initialize() {
            List<string> PortNames = new List<string>();
            int dx;

            btnStopRecv.BringToFront(); 

            cbxField_Delimiter_1.SelectedIndex = 60;
            cbxField_Delimiter_2.SelectedIndex = 0;
            cbxRecord_Delimiter_1.SelectedIndex = 14;
            cbxRecord_Delimiter_2.SelectedIndex = 11;


            gbxComport.Width = 313;
            cbxBaudrate.Text = "9600";
            cbxParity.Text = "None";
            cbxDataBits.Text = "8";
            cbxStopBits.Text = "One";

            cbxComport.Items.Clear();  
            m_task.GetSerialPortNames(ref PortNames);
            for (dx = 0; dx < PortNames.Count; dx++)
                cbxComport.Items.Add(PortNames[dx]);
            if (cbxComport.Items.Count > 0) {
                cbxComport.SelectedIndex = 0;
                m_task.SerialPortSetting(0, 9600, System.IO.Ports.Parity.None, 8, System.IO.Ports.StopBits.One);
            }

            cbx_ENA_AddOn.SelectedIndex = 0;
            cbx_UPC_AddOn.SelectedIndex = 0;
            cbx_Coda_STSP.SelectedIndex = 0;
            cbx_ITF25_Min.SelectedIndex = 0;
            cbx_ITF25_Max.SelectedIndex = cbx_ITF25_Max.Items.Count - 1;
            cbx_C25_Min.SelectedIndex = 0;
            cbx_C25_Max.SelectedIndex = cbx_C25_Max.Items.Count - 1;

            AddRecvEvents();
         }

        private void CommWaitting(bool Waitting) {
            gbxComport.Enabled = !Waitting;
            tbcMain.Enabled = !Waitting;
            changeinfo = Waitting;
        }


        #endregion
                
        #region device settings

        private bool GetDeviceInfo() {
            bool Result = false;
            string[] AutoPowerOff = new string[] { "1 mins", "3 mins", "5 mins", "10 mins", "30 mins", "10 mins", "15 mins", "20 mins", "30 mins" };  

            tvwDevice.Nodes.Clear();
            tvwDevice.Nodes.Add("Forms", "Forms");
            tvwDevice.Nodes.Add("DevInfo", "Devce Information");
            if (m_task.GetDeviceInfo(ref m_devInfo) == 0) {
                for (int dx = 0; dx < m_devInfo.DB.Count; dx++) {
                    TreeNode nNode = tvwDevice.Nodes[0].Nodes.Add("", m_devInfo.DB[dx].DBName);
                    nNode.Nodes.Add("", "RecordSize : " + m_devInfo.DB[dx].Records.ToString());
                    nNode.Nodes.Add("", "Fields : " + m_devInfo.DB[dx].Fields.ToString());
                }

                tvwDevice.Nodes[1].Nodes.Add("", "Device Name : " + m_devInfo.DeviceName);
                tvwDevice.Nodes[1].Nodes.Add("", "F/W Version : " + m_devInfo.DeviceModel);
                tvwDevice.Nodes[1].Nodes.Add("", "Device ID : " + m_devInfo.UnitID);
                tvwDevice.Nodes[1].Nodes.Add("", "Ram Size : " + m_devInfo.RamSize.ToString() + "K");
                tvwDevice.Nodes[1].Nodes.Add("", "Size of Free Ram : " + m_devInfo.SizeofFreeRam.ToString() + "K");
                tvwDevice.Nodes[1].Nodes.Add("", "Main Battery : " + m_devInfo.MainBattery.ToString());
                tvwDevice.Nodes[1].Nodes.Add("", "Back Battery	 : " + m_devInfo.BackBattery.ToString());
                cbxBackLight.SelectedIndex = (int)((m_devInfo.BackLight == false) ? 0 : 1);
                cbxBeep.SelectedIndex = (int)m_devInfo.BuzzerVolume;

                cbxAutoPowerOff.Items.Clear();

                int start = 5;
                int end = 8;

                if (m_devInfo.DeviceName.Length > 5) {
                    if ((m_devInfo.DeviceName.Substring(0, 5) == "Z-107") ||
                    (m_devInfo.DeviceName.Substring(0, 5) == "Z-117")) {
                        start = 0;
                        end = 4;
                    }
                }
                cbxAutoPowerOff.Items.Add("Disable");
                for (int i = start; i <= end; i++) cbxAutoPowerOff.Items.Add(AutoPowerOff[i]);

                if (m_devInfo.AutoPowerOff < cbxAutoPowerOff.Items.Count)
                    cbxAutoPowerOff.SelectedIndex = (int)m_devInfo.AutoPowerOff;
                else cbxAutoPowerOff.SelectedIndex = 0;
                cbxPowerUpScan.SelectedIndex = (int)((m_devInfo.PowerUpScan == false) ? 0 : 1);
                cbxDateFormat.SelectedIndex = (int)m_devInfo.DateModeSetting;
                
                lblComport.Focus();
                Result = true;
            }

            tvwDevice.Nodes[1].ExpandAll();
            
            return Result;
        }

        private void btnGetDeviceSettings_Click(object sender, EventArgs e) {
            CommWaitting(true);
            gbxGetDeviceSettings.Enabled = false;
            if (GetDeviceInfo() == false) MessageBox.Show("Communication fail or Protocol setting error!!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            else {
                chkTimeSync.Checked = false;
                dtpDateTime.Value = DateTime.Now;
                gbxGetDeviceSettings.Enabled = true;
                //To prevent the continuous bond 
                System.Threading.Thread.Sleep(300);
            }
            CommWaitting(false);
        }

        private void chkTimeSync_CheckedChanged(object sender, EventArgs e) {
            dtpDateTime.Enabled = !chkTimeSync.Checked;
        }

        private void cbxBeep_SelectedIndexChanged(object sender, EventArgs e) {
            if (changeinfo == true) return;

            CommWaitting(true);
            if (m_task.SetBuzzerVolume((TaskGenNet.Controller.eBuzzerVolume)cbxBeep.SelectedIndex) != 0)
                MessageBox.Show("Communication fail or Protocol setting error!!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            CommWaitting(false);
        }

        private void cbxAutoPowerOff_SelectedIndexChanged(object sender, EventArgs e) {
            if (changeinfo == true) return;

            CommWaitting(true);
            if (m_task.SetAutoPowerOff((byte)cbxAutoPowerOff.SelectedIndex) != 0)
                MessageBox.Show("Communication fail or Protocol setting error!!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            CommWaitting(false);
        }

        private void cbxPowerUpScan_SelectedIndexChanged(object sender, EventArgs e) {
            if (changeinfo == true) return;

            CommWaitting(true);
            if (m_task.SetPowerUpScan(cbxPowerUpScan.SelectedIndex == 1) != 0)
                MessageBox.Show("Communication fail or Protocol setting error!!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            CommWaitting(false);
        }

        private void cbxDateFormat_SelectedIndexChanged(object sender, EventArgs e) {
            if (changeinfo == true) return;

            CommWaitting(true);
            if (m_task.SetDateDisplayFormat((TaskGenNet.Controller.eDateModeSetting)cbxDateFormat.SelectedIndex) != 0)
                MessageBox.Show("Communication fail or Protocol setting error!!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            CommWaitting(false);
        }

        private void cbxBackLight_SelectedIndexChanged(object sender, EventArgs e) {
            if (changeinfo == true) return;

            CommWaitting(true);
            if (m_task.SetLCDBackLight(cbxBackLight.SelectedIndex == 1) != 0)
                MessageBox.Show("Communication fail or Protocol setting error!!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            CommWaitting(false);
        }

        private void btnGetDeviceTime_Click(object sender, EventArgs e) {
            DateTime nTime = DateTime.Now;

            CommWaitting(true);
            if (m_task.GetDeviceDT(ref nTime) == 0) 
                MessageBox.Show("Device Now Time :\r\n" + nTime.ToString("yyyy/MM/dd HH:mm:ss"), "System Message", MessageBoxButtons.OK, MessageBoxIcon.Information);
            else 
                MessageBox.Show("Get Date/Time fail!!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            CommWaitting(false);
        }

        private void btnSetDeviceTime_Click(object sender, EventArgs e) {
            DateTime settime = ((chkTimeSync.Checked == true) ? DateTime.Now : dtpDateTime.Value);  

            CommWaitting(true);
            if (m_task.SetDeviceDT(settime) != 0) 
                MessageBox.Show("Set Date/Time fail !!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            CommWaitting(false);    
        }

        #endregion
               
        #region barcode settings

        private bool GetBarcodeInfo() {
            bool result = false;
            int temp = 0;

            if (m_task.GetBarcodeInfo(ref BarcodeInfo) == 0) {
                result = true;

                //EAN
                chk_EAN8_Enable.Checked = BarcodeInfo.EAN.ReadEAN8Enable;
                chk_EAN8_CKD.Checked = BarcodeInfo.EAN.SendEAN8CheckDigit;
                chk_EAN13_Enable.Checked = BarcodeInfo.EAN.ReadEAN13Enable;
                chk_EAN13_CKD.Checked = BarcodeInfo.EAN.SendEAN13CheckDigit;
                temp = 0;
                if (BarcodeInfo.EAN.ReadEANwith2DigitAddOn == true) temp += 1;
                if (BarcodeInfo.EAN.ReadEANwith5DigitAddOn == true) temp += 2;
                cbx_ENA_AddOn.SelectedIndex = temp;

                //UPC
                chk_UPCA_Enable.Checked = BarcodeInfo.UPC.ReadUPCAEnable;
                chk_UPCA_Leading.Checked = BarcodeInfo.UPC.SendUPCALeading0;
                chk_UPCA_CKD.Checked = BarcodeInfo.UPC.SendUPCACheckDigit;
                chk_UPCE_Enable.Checked = BarcodeInfo.UPC.ReadUPCEEnable;
                chk_UPCE_Leading.Checked = BarcodeInfo.UPC.SendUPCELeading0;
                chk_UPCE_CKD.Checked = BarcodeInfo.UPC.SendUPCECheckDigit;
                temp = 0;
                if (BarcodeInfo.UPC.ReadUPCwith2DigitAddOn == true) temp += 1;
                if (BarcodeInfo.UPC.ReadUPCwith5DigitAddOn == true) temp += 2;
                cbx_UPC_AddOn.SelectedIndex = temp;
 
                //EAN/UPC
                chk_UE_UPCE_A.Checked = BarcodeInfo.UE.UPCEtoUPCAEnable;
                chk_UE_ISBN.Checked = BarcodeInfo.UE.ISBN_ISSNConversion;
                chk_UE_IATA.Checked = BarcodeInfo.UE.IATACodeEnable;
  
                //C39
                chk_C39_Enable.Checked = BarcodeInfo.CODE39.ReadCode39Enable;
                chk_C39_STSP.Checked = BarcodeInfo.CODE39.SendStartStopChar;
                chk_C39_CKD.Checked = BarcodeInfo.CODE39.SendCheckDigit;
                chk_C39_CDV.Checked = BarcodeInfo.CODE39.CheckDigitVerificationEnable;
                chk_C39_DisFull.Checked = BarcodeInfo.CODE39.DisableFullCode39;
                chk_C39_ITPRA.Checked = BarcodeInfo.CODE39.ITPR_TransmitAChar;
                chk_C39_ITPRC.Checked = BarcodeInfo.CODE39.ITPRConversion;
                chk_C39_Concate.Checked = BarcodeInfo.CODE39.ConcatenationEnable; 
                
                //Codabar
                chk_Coda_Enable.Checked = BarcodeInfo.CODABAR.ReadCodabarEnable;
                chk_Coda_STSPC.Checked = BarcodeInfo.CODABAR.SendStartStopChar;
                chk_Coda_CKD.Checked = BarcodeInfo.CODABAR.SendCheckDigit;
                chk_Coda_CDV.Checked = BarcodeInfo.CODABAR.CheckDigitVerificationEnable;
                temp = 0;
                if (BarcodeInfo.CODABAR.STSP >= 6 && BarcodeInfo.CODABAR.STSP <= 10)
                    cbx_Coda_STSP.SelectedIndex = BarcodeInfo.CODABAR.STSP - 6;
                else cbx_Coda_STSP.SelectedIndex = 0;

                //ITF25
                chk_ITF25_Enable.Checked = BarcodeInfo.ITF25.ReadITF2Of5Enable;
                chk_ITF25_CKD.Checked = BarcodeInfo.ITF25.SendCheckDigit;
                chk_ITF25_CDV.Checked = BarcodeInfo.ITF25.CheckDigitVerificationEnable;
                cbx_ITF25_Min.Text = BarcodeInfo.ITF25.ITF25Min.ToString();
                cbx_ITF25_Max.Text = BarcodeInfo.ITF25.ITF25Max.ToString(); 
                
                //C93
                chk_C93_Enable.Checked = BarcodeInfo.CODE93.ReadCode93Enable;
                chk_C93_STSPC.Checked = BarcodeInfo.CODE93.SendStartStopChar;
                chk_C93_CKD.Checked = BarcodeInfo.CODE93.SendCheckDigit;
                chk_C93_CDV.Checked = BarcodeInfo.CODE93.CheckDigitVerificationEnable;
                chk_C93_DisFull.Checked = BarcodeInfo.CODE93.DisableFullCode93;
 
                //C128
                chk_C128_Enable.Checked = BarcodeInfo.CODE128.ReadCode128Enable;
                chk_C128_CKD.Checked = BarcodeInfo.CODE128.SendCheckDigit;
                chk_C128_FNC2.Checked = BarcodeInfo.CODE128.FNC2Enable;
                chk_C128_CDV.Checked = BarcodeInfo.CODE128.CheckDigitEnable;
 
                //C25
                chk_C25_Enable.Checked = BarcodeInfo.CPC25.ReadCPC2of5Enable;
                chk_C25_CKD.Checked = BarcodeInfo.CPC25.SendCheckDigit;
                chk_C25_CDV.Checked = BarcodeInfo.CPC25.CheckDigitVerificationEnable;
                cbx_C25_Min.Text = BarcodeInfo.CPC25.CPC25Min.ToString();
                cbx_C25_Max.Text = BarcodeInfo.CPC25.CPC25Max.ToString();                                            
            }

            return result;
        }

        private void SetBarcodeInfo() {
            CommWaitting(true);
            if (m_task.SetBarcodeInfo(BarcodeInfo) != 0)
                MessageBox.Show("Set Barcode Information failed !!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            CommWaitting(false);
            System.Threading.Thread.Sleep(150);
            lblComport.Focus();
        }

        private void btnGetBarcodeSettings_Click(object sender, EventArgs e) {
            CommWaitting(true);
            tbcBarcode.Enabled = false;
            if (GetBarcodeInfo() == false) MessageBox.Show("Communication fail or Protocol setting error!!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            else {
                tbcBarcode.Enabled = true;
                //To prevent the continuous bond 
                System.Threading.Thread.Sleep(300);
            }
            CommWaitting(false);       
        }

        private void tbcBarcode_SelectedIndexChanged(object sender, EventArgs e) {
            if (tbcBarcode.SelectedIndex == 9) {
                string[] tamble = new string[2];

                CommWaitting(true);
                btn_UpdatePrePostamble.Enabled = false;
                if (m_task.GetPrePostamble(ref tamble[0], ref tamble[1]) == 0) {
                    btn_UpdatePrePostamble.Enabled = true;
                    txt_Preambe.Text = tamble[0];
                    txt_Postambe.Text = tamble[1];
                }
                else MessageBox.Show("Communication fail or Protocol setting error!!\r\nGet Pre/Postamble Value failed", "System Message", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                CommWaitting(false);
            }
        }

        private void btnProc_UpdatePrePostamble_Click(object sender, EventArgs e) {
            CommWaitting(true);
            if (m_task.SetPrePostamble(txt_Preambe.Text.Trim(), txt_Postambe.Text.Trim()) != 0)
                MessageBox.Show("Set Pre/Postamble fail !!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            CommWaitting(false);
        }

        private void barcode_CheckedChanged(object sender, EventArgs e) {
            CheckBox m_check = (CheckBox)sender;

            switch (m_check.Name.ToString()) {

                #region EAN

                case "chk_EAN8_Enable": //EAN8
                    gbx_ENA8.Enabled = m_check.Checked;
                    BarcodeInfo.EAN.ReadEAN8Enable = m_check.Checked;
                    break;
                case "chk_EAN8_CKD":
                    lbl_ENA8_3.Enabled = m_check.Checked;
                    BarcodeInfo.EAN.SendEAN8CheckDigit = m_check.Checked;
                    break;
                case "chk_EAN13_Enable": //EAN13
                    gbx_ENA13.Enabled = m_check.Checked;
                    BarcodeInfo.EAN.ReadEAN13Enable = m_check.Checked;
                    break;
                case "chk_EAN13_CKD":
                    lbl_ENA13_3.Enabled = m_check.Checked;
                    BarcodeInfo.EAN.SendEAN13CheckDigit = m_check.Checked;
                    break;

                #endregion

                #region UPC

                case "chk_UPCA_Enable": //UPC-A
                    gbx_UPCA.Enabled = m_check.Checked;
                    BarcodeInfo.UPC.ReadUPCAEnable = m_check.Checked;
                    break;
                case "chk_UPCA_Leading":
                    lbl_UPCA_1.Enabled = m_check.Checked;
                    BarcodeInfo.UPC.SendUPCALeading0 = m_check.Checked;
                    break;
                case "chk_UPCA_CKD":
                    lbl_UPCA_4.Enabled = m_check.Checked;
                    BarcodeInfo.UPC.SendUPCACheckDigit = m_check.Checked;
                    break;
                case "chk_UPCE_Enable": //UPC-E
                    gbx_UPCE.Enabled = m_check.Checked;
                    BarcodeInfo.UPC.ReadUPCEEnable = m_check.Checked;
                    break;
                case "chk_UPCE_Leading":
                    lbl_UPCE_1.Enabled = m_check.Checked;
                    BarcodeInfo.UPC.SendUPCELeading0 = m_check.Checked;
                    break;
                case "chk_UPCE_CKD":
                    lbl_UPCE_3.Enabled = m_check.Checked;
                    BarcodeInfo.UPC.SendUPCECheckDigit = m_check.Checked;
                    break;

                #endregion

                #region UE

                case "chk_UE_UPCE_A":  //UE
                    BarcodeInfo.UE.UPCEtoUPCAEnable = m_check.Checked;
                    break;
                case "chk_UE_ISBN":
                    BarcodeInfo.UE.ISBN_ISSNConversion = m_check.Checked;
                    break;
                case "chk_UE_IATA":
                    BarcodeInfo.UE.IATACodeEnable = m_check.Checked;
                    break;

                #endregion

                #region C39

                case "chk_C39_Enable": //C39
                    gbx_C39.Enabled = m_check.Checked;
                    BarcodeInfo.CODE39.ReadCode39Enable = m_check.Checked;
                    break;
                case "chk_C39_STSP":
                    lbl_C39_1.Enabled = m_check.Checked;
                    lbl_C39_4.Enabled = m_check.Checked;
                    BarcodeInfo.CODE39.SendStartStopChar = m_check.Checked;
                    break;
                case "chk_C39_CKD":
                    lbl_C39_3.Enabled = m_check.Checked;
                    BarcodeInfo.CODE39.SendCheckDigit = m_check.Checked;
                    break;
                case "chk_C39_CDV":
                    BarcodeInfo.CODE39.CheckDigitVerificationEnable = m_check.Checked;
                    break;
                case "chk_C39_DisFull":
                    BarcodeInfo.CODE39.DisableFullCode39 = m_check.Checked;
                    break;
                case "chk_C39_ITPRA":
                    BarcodeInfo.CODE39.ITPR_TransmitAChar = m_check.Checked;
                    break;
                case "chk_C39_ITPRC":
                    BarcodeInfo.CODE39.ITPRConversion = m_check.Checked;
                    break;
                case "chk_C39_Concate":
                    BarcodeInfo.CODE39.ConcatenationEnable = m_check.Checked;
                    break;

                #endregion

                #region Codebar

                case "chk_Coda_Enable":
                    gbx_Coda.Enabled = m_check.Checked;
                    BarcodeInfo.CODABAR.ReadCodabarEnable = m_check.Checked;
                    break;
                case "chk_Coda_STSPC":
                    lbl_Coda_1.Enabled = m_check.Checked;
                    lbl_Coda_4.Enabled = m_check.Checked;
                    BarcodeInfo.CODABAR.SendStartStopChar = m_check.Checked;
                    break;
                case "chk_Coda_CKD":
                    lbl_Coda_3.Enabled = m_check.Checked;
                    BarcodeInfo.CODABAR.SendCheckDigit = m_check.Checked;
                    break;
                case "chk_Coda_CDV":
                    BarcodeInfo.CODABAR.CheckDigitVerificationEnable = m_check.Checked;
                    break;

                #endregion

                #region ITF25

                case "chk_ITF25_Enable":
                    gbx_ITF25.Enabled = m_check.Checked;
                    BarcodeInfo.ITF25.ReadITF2Of5Enable = m_check.Checked;
                    break;
                case "chk_ITF25_CKD":
                    lbl_ITF25_2.Enabled = m_check.Checked;
                    BarcodeInfo.ITF25.SendCheckDigit = m_check.Checked;
                    break;
                case "chk_ITF25_CDV":
                    BarcodeInfo.ITF25.CheckDigitVerificationEnable = m_check.Checked;
                    break;

                #endregion

                #region C93

                case "chk_C93_Enable":
                    gbx_C93.Enabled = m_check.Checked;
                    BarcodeInfo.CODE93.ReadCode93Enable = m_check.Checked;
                    break;
                case "chk_C93_STSPC":
                    lbl_C93_1.Enabled = m_check.Checked;
                    lbl_C93_4.Enabled = m_check.Checked;
                    BarcodeInfo.CODE93.SendStartStopChar = m_check.Checked;
                    break;
                case "chk_C93_CKD":
                    lbl_C93_3.Enabled = m_check.Checked;
                    BarcodeInfo.CODE93.SendCheckDigit = m_check.Checked;
                    break;
                case "chk_C93_CDV":
                    BarcodeInfo.CODE93.CheckDigitVerificationEnable = m_check.Checked;
                    break;
                case "chk_C93_DisFull":
                    BarcodeInfo.CODE93.DisableFullCode93 = m_check.Checked;
                    break;

                #endregion

                #region C128

                case "chk_C128_Enable":
                    gbx_C128.Enabled = m_check.Checked;
                    BarcodeInfo.CODE128.ReadCode128Enable = m_check.Checked;
                    break;
                case "chk_C128_CKD":
                    lbl_C128_2.Enabled = m_check.Checked;
                    BarcodeInfo.CODE128.SendCheckDigit = m_check.Checked;
                    break;
                case "chk_C128_FNC2":
                    BarcodeInfo.CODE128.FNC2Enable = m_check.Checked;
                    break;
                case "chk_C128_CDV":
                    BarcodeInfo.CODE128.CheckDigitEnable = m_check.Checked;
                    break;

                #endregion

                #region C25

                case "chk_C25_Enable":
                    gbx_C25.Enabled = m_check.Checked;
                    BarcodeInfo.CPC25.ReadCPC2of5Enable = m_check.Checked;
                    break;
                case "chk_C25_CKD":
                    lbl_C25_2.Enabled = m_check.Checked;
                    BarcodeInfo.CPC25.SendCheckDigit = m_check.Checked;
                    break;
                case "chk_C25_CDV":
                    BarcodeInfo.CPC25.CheckDigitVerificationEnable = m_check.Checked;
                    break;

                #endregion
                    
            }
            if (changeinfo == false) SetBarcodeInfo();
        }

        private void barcodeBox_SelectedIndexChanged(object sender, EventArgs e) {
            ComboBox m_cbx = (ComboBox)sender;
            int nIndex = m_cbx.SelectedIndex;
            Image[] img = new Image[4] { null, Properties.Resources.EAN2, Properties.Resources.EAN5, Properties.Resources.EAN5 };
            byte[] Nibble = new byte[5] { 0x0a, 0x09, 0x08, 0x07, 0x06 };
            
            switch (m_cbx.Name.ToString()) {
                case "cbx_ENA_AddOn":
                    BarcodeInfo.EAN.ReadEANwith2DigitAddOn = (nIndex % 2 != 0);
                    BarcodeInfo.EAN.ReadEANwith5DigitAddOn = (nIndex >= 2);
                    pcxProc_ENA8.BackgroundImage = img[nIndex];
                    pcxProc_ENA13.BackgroundImage = img[nIndex];
                    break;
                case "cbx_UPC_AddOn":
                    BarcodeInfo.UPC.ReadUPCwith2DigitAddOn = (nIndex % 2 != 0);
                    BarcodeInfo.UPC.ReadUPCwith5DigitAddOn = (nIndex >= 2);
                    pcxProc_UPCA.BackgroundImage = img[nIndex];
                    pcxProc_UPCE.BackgroundImage = img[nIndex];
                    break;
                case "cbx_Coda_STSP":
                    BarcodeInfo.CODABAR.STSP = Nibble[nIndex];
                    break;
                case "cbx_ITF25_Min":
                    BarcodeInfo.ITF25.ITF25Min = byte.Parse(m_cbx.Text);
                    break;
                case "cbx_ITF25_Max":
                    BarcodeInfo.ITF25.ITF25Max = byte.Parse(m_cbx.Text);
                    break;
                case "cbx_C25_Min":
                    BarcodeInfo.CPC25.CPC25Min = byte.Parse(m_cbx.Text);
                    break;
                case "cbx_C25_Max":
                    BarcodeInfo.CPC25.CPC25Max = byte.Parse(m_cbx.Text);
                    break;
                
            }
                        
            if (changeinfo == false) SetBarcodeInfo();
        }
                


        #endregion
               
        #region receive data

        private bool GetRecvInfo() {
            bool Result = false;
            
            m_Record.Clear();
            m_Field.Clear();
            cbxFormName.Items.Clear();
                        
            if (m_task.GetDeviceInfo(ref m_devInfo) == 0) {
                for (int dx = 0; dx < m_devInfo.DB.Count; dx++) {
                    m_Record.Add(m_devInfo.DB[dx].Records);
                    m_Field.Add(m_devInfo.DB[dx].Fields);
                    cbxFormName.Items.Add(m_devInfo.DB[dx].DBName);
                }
                
                Result = true;
            }
            if (cbxFormName.Items.Count > 0) cbxFormName.SelectedIndex = 0;

            return Result;
        }

        private void AddRecvEvents() {
            if (m_task != null) {
                RemoveRecvEvents();
                m_task.PDXEvent += new TaskGenNet.Controller.PDXRecvEventHandler(OnPDXDataArrived);
                m_task.PDXCompleted += new TaskGenNet.Controller.PDXRecvCompleted(OnDataRecvCompleted);
            }
        }

        private void RemoveRecvEvents()
        {
            if (m_task != null)
            {
                m_task.PDXEvent -= new TaskGenNet.Controller.PDXRecvEventHandler(OnPDXDataArrived);
                m_task.PDXCompleted -= new TaskGenNet.Controller.PDXRecvCompleted(OnDataRecvCompleted);
            }
        }

        #region OnPDXDataArrived

        private delegate void InvokePDXDataArrived(string PDX, bool Header);

        private void OnPDXDataArrived(string PDX, bool Header)
        {
            PDXDataArrived(PDX, Header);
        }

        private void PDXDataArrived(string PDX, bool Header)
        {
            if (this.InvokeRequired)
                this.Invoke(new InvokePDXDataArrived(PDXDataArrived), new object[] { PDX, Header });
            else
            {
                int FieldCount = int.Parse(txtUpload_Fields.Text);
                int nCnt = 0;

                if (btnStopRecv.Visible == false) btnStopRecv.Visible = true;
                if (Header == true)
                {
                    ListViewItem lvw = lvwUpload.Items.Add("H");
                    lvw.SubItems.Add(PDX);
                }
                else
                {
                    if (tsbMain.Value + 1 <= tsbMain.Maximum) tsbMain.Value++;
                    strTemplete += PDX + "\r";
                    if (tsbMain.Value % FieldCount == 0)
                    {
                        nCnt = (int)tsbMain.Value / FieldCount;
                        ListViewItem lvw = lvwUpload.Items.Add(nCnt.ToString());
                        string[] xxTemp;
                        string nStemp = strTemplete.Substring(0, strTemplete.Length - 1);

                        txtRecvCnt.Text = nCnt.ToString();
                        xxTemp = nStemp.Split(new string[] { "\r" }, StringSplitOptions.None);
                        for (int dx = 0; dx < xxTemp.Length; dx++) lvw.SubItems.Add(xxTemp[dx]);
                        lvw.EnsureVisible();
                        strTemplete = "";
                    }
                }
            }
        }

        #endregion

        #region OnDataRecvCompleted

        private delegate void InvokeDataRecvCompleted(bool Result);

        public void OnDataRecvCompleted(bool Result)
        {
            DataRecvCompleted(Result);
        }

        private void DataRecvCompleted(bool Result)
        {
            if (this.InvokeRequired)
                this.Invoke(new InvokeDataRecvCompleted(DataRecvCompleted), new object[] { Result });
            else
            {
                tsbMain.Value = 0;
                btnStopRecv.Visible = false;
                if (Result == false)
                    MessageBox.Show("Receive Data fail !!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.None);
                btnUploadData_Save.Enabled = ((lvwUpload.Items.Count > 0) ? true : false);
                btnGetDeviceSettings.Enabled = true;
                CommWaitting(false);
            }
        }

        #endregion

        private void btnGetData_Click(object sender, EventArgs e) {
            CommWaitting(true);
            gbxReceiveData.Enabled = false;
            if (GetRecvInfo() == false) MessageBox.Show("Communication fail or Protocol setting error!!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            else {
                gbxReceiveData.Enabled = true;
                //To prevent the continuous bond 
                System.Threading.Thread.Sleep(300);
            }
            CommWaitting(false);
        }

        private void cbxFormName_SelectedIndexChanged(object sender, EventArgs e) {
            txtUpload_Records.Text = m_Record[cbxFormName.SelectedIndex].ToString();
            txtUpload_Fields.Text = m_Field[cbxFormName.SelectedIndex].ToString();
            btnUploadData_Save.Enabled = false;
            btnUploadData.Enabled = (m_Record[cbxFormName.SelectedIndex] > 0) ? true : false;
            btnDeleteData.Enabled = btnUploadData.Enabled;
            if (int.Parse(txtUpload_Fields.Text) > 0) {
                lvwUpload.Columns.Clear();
                lvwUpload.Columns.Add("Index", 60);
                for (int dx = 0; dx < int.Parse(txtUpload_Fields.Text); dx++)
                    lvwUpload.Columns.Add("Field_" + (dx + 1).ToString(), 200);
            }
        }

        private void btnUploadData_Click(object sender, EventArgs e)
        {
            CommWaitting(true);
            strTemplete = "";
            txtRecvCnt.Text = "0";
            lvwUpload.Items.Clear();
            tsbMain.Value = 0;
            tsbMain.Maximum = (int)(int.Parse(txtUpload_Records.Text) * int.Parse(txtUpload_Fields.Text));
            if (m_task.GetPDX(cbxFormName.SelectedIndex) == -1)
            {
                CommWaitting(false);
                MessageBox.Show("Port open fail !!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        private void btnDeleteData_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure to delete data [" + cbxFormName.Text + "] !!", "System Message", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                CommWaitting(true);
                if (m_task.ClearDataBuffer((int)cbxFormName.SelectedIndex) == 0)
                {
                    btnDeleteData.Enabled = false;
                    btnUploadData.Enabled = false;
                    txtUpload_Records.Text = "0";
                    m_Record.RemoveAt(cbxFormName.SelectedIndex);
                    m_Record.Insert(cbxFormName.SelectedIndex, 0);
                }
                else MessageBox.Show("Delete Data fail !!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                CommWaitting(false);
            }
        }

        private void btnStopRecv_Click(object sender, EventArgs e)
        {
            m_task.StopReceivePDX();
        }

        private void btnUploadData_Save_Click(object sender, EventArgs e)
        {
            SaveFileDialog newSaveFileDialog = new SaveFileDialog();
            int Start = (chkFileHeader.Checked == true) ? 0 : 1;
            string nField = "";
            string nRecord = "";
            ComboBox[] nBox = new ComboBox[4];

            nBox[0] = cbxField_Delimiter_1;
            nBox[1] = cbxField_Delimiter_2;
            nBox[2] = cbxRecord_Delimiter_1;
            nBox[3] = cbxRecord_Delimiter_2;

            for (int dx = 0; dx < 4; dx++)
            {
                switch (nBox[dx].SelectedIndex)
                {
                    case 0:
                        if (dx >= 2) nRecord += "";
                        else nField += "";
                        break;
                    default:
                        if (dx >= 2) nRecord += Convert.ToChar((nBox[dx].SelectedIndex - 1));
                        else nField += Convert.ToChar((nBox[dx].SelectedIndex - 1));
                        break;
                }
            }

            newSaveFileDialog.FileName = "BarcodeData.txt";
            newSaveFileDialog.Filter = "TEXT File(*.txt)|*.txt";
            if (newSaveFileDialog.ShowDialog() != DialogResult.Cancel)
            {
                System.IO.StreamWriter wr = new System.IO.StreamWriter(newSaveFileDialog.FileName, false, System.Text.UnicodeEncoding.Default);
                for (int dx = Start; dx < lvwUpload.Items.Count; dx++)
                {
                    if (dx == 0)
                    {
                        if (chkFileHeader.Checked == true) wr.Write(lvwUpload.Items[dx].SubItems[1].Text + nRecord);
                    }
                    else
                    {
                        string fr = "";
                        for (int dy = 1; dy < lvwUpload.Columns.Count; dy++)
                            fr += lvwUpload.Items[dx].SubItems[dy].Text + ((dy != lvwUpload.Columns.Count - 1) ? nField : "");
                        wr.Write(fr + nRecord);
                    }
                }
                wr.Close();
                wr = null;

                MessageBox.Show("Save file " + newSaveFileDialog.FileName + "  completed !!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }

            newSaveFileDialog.Dispose();
            newSaveFileDialog = null;


        }


        #endregion
                
        #region free task download

        #region OnFreeTaskProgress

        private delegate void InvokeFreeTaskProgress(int Max, int Value, bool End, int result);

        public void OnFreeTaskProgress(int Max, int Value, bool End, int result)
        {
            FreeTaskProgress(Max, Value, End, result);
        }

        private void FreeTaskProgress(int Max, int Value, bool End, int result) {
            if (this.InvokeRequired)
                this.Invoke(new InvokeFreeTaskProgress(FreeTaskProgress), new object[] { Max, Value, End, result });
            else {
                string msg = "";

                tsbMain.Maximum = Max;
                tsbMain.Value = Value;
                if (End == true) {
                    CommWaitting(false);
                    if (result == 0) MessageBox.Show("Download free task completed !!\r\n\r\nWaitting for device to create database !!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    else {
                        switch (result) {
                            case -1: msg = "Port open failed !!"; break;
                            case -2: msg = "File not found !!"; break;
                            case -3: msg = "The specified free task is empty file !!"; break;
                            case -4: msg = "Download free task failed !!"; break;
                            case -5: msg = "Communication fail or protocol setting error!!"; break;
                        }
                        MessageBox.Show(msg, "System Message", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    }
                    tsbMain.Value = 0;
                    RemoveFreeTaskEvent();
                    this.Refresh();
                }
            }
        }

        #endregion

        private void AddFreeTaskEvent()
        {
            if (m_task != null)
            {
                m_task.FreeTaskEvent += new TaskGenNet.Controller.FreeTaskProgress(OnFreeTaskProgress);
            }
        }

        private void RemoveFreeTaskEvent()
        {
            if (m_task != null)
            {
                m_task.FreeTaskEvent -= new TaskGenNet.Controller.FreeTaskProgress(OnFreeTaskProgress);
            }
        }

        private void btnDownloadFree_Click(object sender, EventArgs e)
        {
            OpenFileDialog newOpenFileDialog = new OpenFileDialog();
            
            newOpenFileDialog.Filter = "Free Task File(*.TSF)|*.TSF";
            newOpenFileDialog.Multiselect = false;
            if (newOpenFileDialog.ShowDialog() != DialogResult.Cancel) {
                CommWaitting(true);
                RemoveFreeTaskEvent(); //Clear Stack
                AddFreeTaskEvent();    //Add Event
                m_task.DownloadTSF(newOpenFileDialog.FileName);
            }

            newOpenFileDialog.Dispose();
            newOpenFileDialog = null;
        }

        #endregion

        #region procedure task download

        #region OnProceTaskProgress

        private delegate void InvokeProceTaskProgress(int Max, int Value, bool End, bool Result, string Msg);

        public void OnProceTaskProgress(int Max, int Value, bool End, bool Result, string Msg)
        {
            ProceTaskProgress(Max, Value, End, Result, Msg);
        }

        private void ProceTaskProgress(int Max, int Value, bool End, bool Result, string Msg)
        {
            if (this.InvokeRequired)
                this.Invoke(new InvokeProceTaskProgress(ProceTaskProgress), new object[] { Max, Value, End, Result, Msg });
            else {
                tsbMain.Maximum = Max;
                tsbMain.Value = ((Value <= tsbMain.Maximum) ? Value : tsbMain.Maximum);                
                lblMsg.Text = Msg;
                if (End == true) {
                    CommWaitting(false);
                    if (Result == true) MessageBox.Show("Procedure Task Download Completed !!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.None);
                    else MessageBox.Show("Procedure Task Download fail !!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    tsbMain.Value = 0;
                    RemoveProcedureTaskEvent();
                    this.Refresh();
                }
            }
        }

        #endregion

        private void AddProcedureTaskEvent()
        {
            if (m_task != null)
            {
                m_task.ProceTaskEvent += new TaskGenNet.Controller.ProceTaskProgress(OnProceTaskProgress);
            }
        }

        private void RemoveProcedureTaskEvent()
        {
            if (m_task != null)
            {
                m_task.ProceTaskEvent -= new TaskGenNet.Controller.ProceTaskProgress(OnProceTaskProgress);
            }
        }

        private bool DeviceExistData(ref bool DataExist) {
            TaskGenNet.Controller.uDeviceInfo xSystem = new TaskGenNet.Controller.uDeviceInfo();                    
            bool result = false;

            DataExist = false;
            if (m_task.GetDeviceInfo(ref xSystem) == 0) {
                result = true;
                for (int dx = 0; dx < xSystem.DB.Count; dx++) {
                    if (xSystem.DB[dx].Records > 0) {
                        DataExist = true;
                        break;
                    }
                }
            }
            
            return result;
        }

        private void btnDownloadProcedure_Click(object sender, EventArgs e) {
            OpenFileDialog newOpenFileDialog = new OpenFileDialog();
            bool StartDownload = true;
            bool DataExist = false;

            newOpenFileDialog.Filter = "Procedure task link file(*.TSL)|*.TSL";
            newOpenFileDialog.Multiselect = false;
            if (newOpenFileDialog.ShowDialog() != DialogResult.Cancel) {                
                CommWaitting(true);
                if (DeviceExistData(ref DataExist) == true)  {
                    if (DataExist == true && MessageBox.Show("Data exists, continue to download ?", "System Message", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes) StartDownload = false;
                    if (StartDownload == true) {
                        RemoveProcedureTaskEvent(); //Clear Stack
                        AddProcedureTaskEvent();    //Add Event
                        m_task.DownloadTSL(newOpenFileDialog.FileName, true);
                    }
                }
                else MessageBox.Show("Communication failure , please check the equipment !!", "System Message", MessageBoxButtons.OK, MessageBoxIcon.Question);
            }

            newOpenFileDialog.Dispose();
            newOpenFileDialog = null;
        }

        #endregion

        

        
        

        
        









    }
}